(** Translate parse tree into the intermediate abstract syntax tree *)

open Misc
open Parsetree
open Exp
open Symbtree

exception Undef_non_pure_expression 
exception Conversion_error of string * Location.t

(** [p_exp_to_can_exp pe] returns a {!can_exp} representing [pe].   
   Throws [Undef_non_pure_expression] if it contains indirections. *)
val p_exp_to_can_exp : p_expression -> can_exp

(** [p_exp_to_can_exp_se pe stmts] returns the read side-effects required for the indirections in [pe]
   and a {!can_exp} representing [pe]*)
val p_exp_to_can_exp_se : p_expression -> can_stmt list -> can_stmt list * can_exp

(** [p_exp_list_to_can_exp_list_se pel stmts] returns the read side-effects required for the indirections in [pel]
   and a {!can_exp list} representing [pel]*)
val p_exp_list_to_can_exp_list_se : p_expression list -> can_stmt list -> can_stmt list * can_exp list

(** [p_exp_to_can_prop_se pexp] returns the read side-effects
  required to evaluate [pexp] appended to [stmts] and a proposition
  to evaluate. *)
val p_exp_to_can_prop_se : p_expression -> can_stmt list * can_stmt list
val p_exp_to_can_prop : p_expression -> can_prop * can_prop

val prop_to_can_prop : a_proposition -> Location.t -> can_prop
val prop_to_ip_body : a_proposition -> Location.t -> (can_pure * can_form) list 

