(** Miscellaneous useful types and functions. *)

(** {2 Miscellaneous operations} *)

module StringSet : Set.S with type elt = string
module StringMap : Map.S with type key = string

val iter_pairs :  ('a * 'b -> unit) -> 'a list -> 'b list -> unit
val iter_pairwise : ('a * 'a -> unit) -> 'a list -> unit
val map_partial: ('a -> 'b option) -> 'a list -> 'b list
val map_option : ('a -> 'b option) -> 'a list -> 'b list option
val unSome : 'a option -> 'a

(** {2 Identifiers} *)

type ident

val compare_ident : ident -> ident -> int
val abs_compare_ident : ident -> ident -> int
val equal_ident : ident -> ident -> bool

module IdSet : Set.S with type elt = ident
module IdMap : Map.S with type key = ident
val init_gensym : unit -> unit
val gensym : ident -> ident
val gensym_garb: ident -> ident
val create_ident : string -> ident
val is_existential : ident -> bool
val is_garbage : ident -> bool
val un_existential : ident -> ident
val into_existential : ident -> ident
val string_of_ident : ident -> string
val pp_ident : Format.formatter -> ident -> unit
val pp_idset : Format.formatter -> IdSet.t -> unit
val tempid : unit -> ident

(** Identifier corresponding to the variable "TID" *)
val tid_var : ident

(** Identifier corresponding to the variable "myTID" *)
val mytid_var : ident
