
let wp lex =
  let iteml =
    try
      Parser.program Lexer.token lex 
    with 
      | Location.Parse_error(s,loc) -> 
	  print_endline (Location.sprint loc ^ s);
	  invalid_arg ("") in
  Vcgen.vcgen iteml

let verify_wp lex =
  Misc.init_gensym ();
  let fmt = !Config.formatter in
  let valid = ref true in
  let prover = (*if !Config.prover_num = 2 then Prover2.prover else*) Prover.default_prover in
  let abstraction = Abstraction.mk_abstraction prover in
  try 
    let (ht,ril,iteml) = wp lex in
    let init_time = Sys.time() in
    let reps = !Config.repetitions in
    for i=1 to reps do valid := (Symbsimp.check_props prover abstraction ht ril iteml) done;
    let avg_time = (Sys.time () -. init_time) /. (float reps) in
    if !Config.print_stats then
      begin
        Format.fprintf fmt "@.Stabilisation iterations: %d" (!Symbsimp.stabilisation_iterations/reps);
        Format.fprintf fmt "@.Theorem prover calls (stabilisation): %d@." (!Abstraction.prover_calls/reps);
        Format.fprintf fmt "Execution time: %f@." avg_time;
      end; 
    Format.fprintf fmt "@.%sValid@." (if !valid then "" else "NOT ")
 with Invalid_argument s -> Format.pp_print_string fmt s

let verify_wp_string s =
  let lex = Lexing.from_string s
  in verify_wp lex

let verify_wp_fname fname =
  let ic = if fname = "-" then stdin else open_in fname in
  let lex = Lexing.from_channel ic in
  Lexer.init lex fname;
  verify_wp lex;
  close_in ic
